BossPanel_Quests = BossPanelPlugin:new({
	name          = BossPanel_QuestsLocals.NAME,
	description   = BossPanel_QuestsLocals.DESCRIPTION,
	version       = "0.1.6",
	releaseDate   = "04-01-2006",
	aceCompatible = 103,
	bpCompatible  = 095,
	author        = "Kemayo",
	email         = "kemayo@gmail.com",
	website       = "http://kemayo.wowinterface.com/",
	category      = "quests",
	db            = AceDatabase:new("BossPanel_QuestsDB"),
	defaults      = {
		showLevels = TRUE,
		showDifficulty = TRUE,
		showArea = TRUE,
		showCurrentAreaOnly = FALSE,
		showCurrentAreaDescriptionOnly = FALSE,
		showDescription = TRUE,
		showCompletedObjectives = FALSE,
		showInItemTooltip = TRUE,
		colorObjectives = TRUE,
	},
	charDefaults  = {
		hidden = {}
	},
	cmd           = AceChatCmd:new(BossPanel_QuestsLocals.COMMANDS, BossPanel_QuestsLocals.CMD_OPTIONS),
	loc           = BossPanel_QuestsLocals,
	hasIcon       = TRUE,
	canHideText   = TRUE,
	overrideTooltip = TRUE,
	
	-- Methods
	
	--Credit to TheFly
	ShowTooltip = function(self)
		self:OpenChildFrame(BossPanel_QuestsFrame)
		self.idleTime = 0
		self.allowedIdle = 10
		self.enteredFrame = nil
	end,
	
	--Credit to TheFly
	Frame_OnUpdate = function(self)
		if not self.enteredFrame then
			self.idleTime = self.idleTime + arg1
			if self.idleTime >= self.allowedIdle then
				BossPanel_QuestsFrame:Hide()
			end
		end
	end,
	
	--Credit to TheFly
	Quest_Enter = function(self)
		self.enteredFrame = TRUE
		self.allowedIdle = 2
	end,
	
	--Credit to TheFly
	Quest_Leave = function(self)
		self.enteredFrame = FALSE
	end,
	
	--Credit to TheFly
	Quest_Click = function(self)
		self.idleTime = 0
		if this:GetID()>20 then
			local zone = self.quests[this:GetID()-20].zone
			self.charData.hidden[zone] = not self.charData.hidden[zone]
			self:UpdateTooltip()
		else
			local id = this:GetID()
			local questIndex
			for i, zone in self.quests do
				if not zone.hide then
					for ii, quest in zone.quests do
						id = id-1
						if id==0 then
							questIndex = quest.id
						end
					end
				end
			end
			
			if IsShiftKeyDown() then
				-- Shift-click toggles quest-watch on this quest.
				if ( IsQuestWatched(questIndex) ) then
					RemoveQuestWatch(questIndex);
					QuestWatch_Update();
				else
					-- Set error if no objectives
					if ( GetNumQuestLeaderBoards(questIndex) == 0 ) then
						UIErrorsFrame:AddMessage(QUEST_WATCH_NO_OBJECTIVES, 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
						return;
					end
					-- Set an error message if trying to show too many quests
					if ( GetNumQuestWatches() >= MAX_WATCHABLE_QUESTS ) then
						UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, MAX_WATCHABLE_QUESTS), 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
						return;
					end
					AddQuestWatch(questIndex);
					QuestWatch_Update();
				end
			else
				if QuestLogFrame:IsVisible() then
					if self.lastIndex == questIndex then
						HideUIPanel(QuestLogFrame)
					end
				else
					ShowUIPanel(QuestLogFrame)
				end
				if (self.numEntries > QUESTS_DISPLAYED) then
					if (questIndex < self.numEntries - QUESTS_DISPLAYED) then
						FauxScrollFrame_SetOffset(QuestLogListScrollFrame, questIndex - 1);
						QuestLogListScrollFrameScrollBar:SetValue((questIndex - 1) * QUESTLOG_QUEST_HEIGHT);
					else
						FauxScrollFrame_SetOffset(QuestLogListScrollFrame, self.numEntries - QUESTS_DISPLAYED);
						QuestLogListScrollFrameScrollBar:SetValue((self.numEntries - QUESTS_DISPLAYED) * QUESTLOG_QUEST_HEIGHT);
					end
				end
				
				SelectQuestLogEntry(questIndex)
				QuestLog_SetSelection(questIndex)
				self.lastIndex = questIndex
			end
		end
	end,
	
	IsShowingLevels = function(self)
		return self.data.showLevels
	end,
	
	ToggleShowingLevels = function(self, silent)
		self.data.showLevels = not self.data.showLevels
		return self.data.showLevels
	end,
	
	IsShowingDifficulty = function(self)
		return self.data.showDifficulty
	end,
	
	ToggleShowingDifficulty = function(self, silent)
		self.data.showDifficulty = not self.data.showDifficulty
		return self.data.showDifficulty
	end,
	
	IsShowingArea = function(self)
		return self.data.showArea
	end,
	
	ToggleShowingArea = function(self, silent)
		self.data.showArea = not self.data.showArea
		return self.data.showArea
	end,
	
	IsShowingCurrentAreaOnly = function(self)
		return self.data.showCurrentAreaOnly
	end,
	
	ToggleShowingCurrentAreaOnly = function(self, silent)
		self.data.showCurrentAreaOnly = not self.data.showCurrentAreaOnly
		return self.data.showCurrentAreaOnly
	end,
	
	IsShowingCurrentAreaDescriptionOnly = function(self)
		return self.data.showCurrentAreaDescriptionOnly
	end,
	
	ToggleShowingCurrentAreaDescriptionOnly = function(self, silent)
		self.data.showCurrentAreaDescriptionOnly = not self.data.showCurrentAreaDescriptionOnly
		return self.data.showCurrentAreaDescriptionOnly
	end,
	
	IsShowingDescription = function(self)
		return self.data.showDescription
	end,
	
	ToggleShowingDescription = function(self, silent)
		self.data.showDescription = not self.data.showDescription
		return self.data.showDescription
	end,
	
	IsColoringObjectives = function(self)
		return self.data.colorObjectives
	end,
	
	ToggleColoringObjectives = function(self, silent)
		self.data.colorObjectives = not self.data.colorObjectives
		return self.data.colorObjectives
	end,
	
	IsShowingCompletedObjectives = function(self)
		return self.data.showCompletedObjectives
	end,
	
	ToggleShowingCompletedObjectives = function(self, silent)
		self.data.showCompletedObjectives = not self.data.showCompletedObjectives
		return self.data.showCompletedObjectives
	end,
	
	IsShowingInItemTooltip = function(self)
		return self.data.showInItemTooltip
	end,
	
	ToggleShowingInItemTooltip = function(self, silent)
		self.data.showInItemTooltip = not self.data.showInItemTooltip
		return self.data.showDescription
	end,
	
	Initialize = function(self)
		--In case people are upgrading... refresh any options which aren't set with the defaults.
		if self.data.version ~= self.version then
			if self.data.showLevels==nil then self.data.showLevels = self.defaults.showLevels end
			if self.data.showDifficulty==nil then self.data.showDifficulty = self.defaults.showDifficulty end
			if self.data.showArea==nil then self.data.showArea = self.defaults.showArea end
			if self.data.showCurrentAreaOnly==nil then self.data.showCurrentAreaOnly = self.defaults.showCurrentAreaOnly end
			if self.data.showCurrentAreaDescriptionOnly==nil then self.data.showCurrentAreaDescriptionOnly = self.defaults.showCurrentAreaDescriptionOnly end
			if self.data.showDescription==nil then self.data.showDescription = self.defaults.showDescription end
			if self.data.showCompletedObjectives==nil then self.data.showCompletedObjectives = self.defaults.showCompletedObjectives end
			if self.data.showInItemTooltip==nil then self.data.showInItemTooltip = self.defaults.showInItemTooltip end
			if self.data.colorObjectives==nil then self.data.colorObjectives = self.defaults.colorObjectives end
		end
		
		self.numQuests = 0
		self.numEntries = 0
		self.quests = {}
		self.items = {}
		self.mobs = {}
		self.firstQuestLogUpdate = FALSE
		self.timeSinceLogin = 0
		
		--Credit to TheFly
		BossPanel_QuestsFrameText:SetText(BossPanel_QuestsLocals.TEXT_QUESTS)
	end,
	
	Enable = function(self)
		self:RegisterEvent("QUEST_LOG_UPDATE", "Update")
		self:RegisterEvent("PLAYER_LEVEL_UP", "Update")
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "OnMouseOverUnit")
		self:Hook("ContainerFrameItemButton_OnEnter", "OnItemTooltip")
	end,
	
	OnUpdate = function(self, timeSinceLast)
		--If the quest log is scanned too early, the previous character's quests will be retrieved.
		--Or, if the current character has no savedvariables, it'll just get nothing.  Sometimes.
		--Todo: Convert this to 
		self.timeSinceLogin = self.timeSinceLogin + timeSinceLast
		if self.timeSinceLogin >= 10 then
            self.firstQuestLogUpdate = TRUE
			self:Update()
			self:StopOnUpdate()
		end
	end,
	
	OnMouseOverUnit = function(self)
		-- Make sure that this is wanted, and that the tooltip is actually read/writeable.
		if self:IsShowingInItemTooltip() and self:CanMessWithGameTooltip() then
			--Comes with color codes.  Strip 'em out.
			local thisMob = string.gsub(getglobal('GameTooltipTextLeft1'):GetText(),"|c........(.*)|?r?","%1")
			if self.mobs[thisMob] then
				self:AddToGameTooltipAndFixHeight(self.mobs[thisMob].quest..": "..self.mobs[thisMob].needed, self.mobs[thisMob].color)
			end
		end
	end,
	
	OnItemTooltip = function(self)
		self.Hooks["ContainerFrameItemButton_OnEnter"].orig()
		if self:IsShowingInItemTooltip() and self:CanMessWithGameTooltip() then
			--Comes with color codes.  Strip 'em out.
			local thisItem = string.gsub(getglobal('GameTooltipTextLeft1'):GetText(),"|c........(.*)|?r?","%1")
			if self.items[thisItem] then
				self:AddToGameTooltipAndFixHeight(self.items[thisItem].quest..": "..self.items[thisItem].needed, self.items[thisItem].color)
			end
		end
	end,
	
	CanMessWithGameTooltip = function(self)
		if GameTooltip and getglobal('GameTooltipTextLeft1'):IsVisible() and getglobal('GameTooltipTextLeft1'):GetText() ~= nil then
			return TRUE
		else
			return FALSE
		end
	end,
	
	AddToGameTooltipAndFixHeight = function(self,text,color)
		GameTooltip:AddLine(text, color.r, color.g, color.b)
		
		-- Fix up the width of the tooltip
		length = getglobal(GameTooltip:GetName() .. "TextLeft" .. GameTooltip:NumLines()):GetStringWidth()
		-- Space for right-border:
		length = length + 22
		
		GameTooltip:SetHeight(GameTooltip:GetHeight() + 14)
		if length > GameTooltip:GetWidth() then
			GameTooltip:SetWidth(length)
		end
	end,
	
	MenuSettings = function(self)
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_LEVELS,
			func = function()
				self:ToggleShowingLevels()
			end,
			checked = self:IsShowingLevels(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_DIFFICULTY,
			func = function()
				self:ToggleShowingDifficulty()
			end,
			checked = self:IsShowingDifficulty(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_AREA,
			func = function()
				self:ToggleShowingArea()
			end,
			checked = self:IsShowingArea(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_CURRENT_AREA_ONLY,
			func = function()
				self:ToggleShowingCurrentAreaOnly()
			end,
			checked = self:IsShowingCurrentAreaOnly(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_DESCRIPTION,
			func = function()
				self:ToggleShowingDescription()
			end,
			checked = self:IsShowingDescription(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_COLOR_OBJECTIVES,
			func = function()
				self:ToggleColoringObjectives()
			end,
			checked = self:IsColoringObjectives(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_COMPLETED_OBJECTIVES,
			func = function()
				self:ToggleShowingCompletedObjectives()
			end,
			checked = self:IsShowingCompletedObjectives(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_CURRENT_AREA_DESCRIPTION_ONLY,
			func = function()
				self:ToggleShowingCurrentAreaDescriptionOnly()
			end,
			checked = self:IsShowingCurrentAreaDescriptionOnly(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_IN_ITEM_TOOLTIP,
			func = function()
				self:ToggleShowingInItemTooltip()
			end,
			checked = self:IsShowingInItemTooltip(),
			keepShownOnClick = TRUE,
		})
	end,
	
	UpdateData = function(self)
		if self.firstQuestLogUpdate then
			--UpdateText and UpdateTooltip both need to know things about the questlog.  Better to do this once.
			local startingQuestLogSelection = GetQuestLogSelection()
			local numEntries, numQuests = GetNumQuestLogEntries()
			local quests = {}
			local items = {}
			local mobs = {}
			local zoneIndex = 0
			local quest
			
			if numEntries > 0 then
				for quest = 1, numEntries, 1 do
					SelectQuestLogEntry(quest)
					local strQuestLogTitleText, strQuestLevel, strQuestTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(quest)
					local strQuestDescription, strQuestObjectives = GetQuestLogQuestText()
					local q = {}
					local thisQuest = "  "
					local strQuestLevelTag = ""
					
					if strQuestTag == self.loc.ELITE then
						strQuestLevelTag = "+"
					elseif strQuestTag == self.loc.DUNGEON then
						strQuestLevelTag = "D"
					elseif strQuestTag == self.loc.PVP then
						strQuestLevelTag = "P"
					elseif strQuestTag == self.loc.RAID then
						strQuestTag = "R"
					end
					
					if isHeader then
						zoneIndex = zoneIndex + 1
						quests[zoneIndex] = { zone=strQuestLogTitleText, quests={} }
					else
						if isComplete == 1 then
							isComplete = format("(%s)",self.loc.QUEST_DONE)
						elseif isComplete == -1 then
							isComplete = format("(%s)",self.loc.QUEST_FAILED)
						else
							isComplete = nil
						end
						
						q.title = strQuestLogTitleText
						q.level = strQuestLevel
						q.tag = strQuestLevelTag
						q.complete = isComplete
						q.objective = strQuestObjectives
						
						local lb = {}
						if (GetNumQuestLeaderBoards() > 0) then
							for ii=1, GetNumQuestLeaderBoards() do
								local desc, qtype, done = GetQuestLogLeaderBoard(ii)
								
								if qtype == 'item' then
									itemName,numItems,numNeeded = BossPanelUtils.Deformat (desc, QUEST_OBJECTS_FOUND)
									desc = itemName
								elseif qtype == 'monster' then
									mobName,numItems,numNeeded = BossPanelUtils.Deformat (desc, QUEST_MONSTERS_KILLED)
									if mobName == nil or numItems == nil or numNeeded == nil then
										--Sometimes we get objectives like "Find Mankrik's Wife: 0/1", which are listed as "monster".
										mobName, numItems, numNeeded = BossPanelUtils.Deformat (desc, QUEST_OBJECTS_FOUND)
									end
									desc = mobName
								end
								
								local color = {}
								if numItems ~= nil then
									-- This quest involves items!
									color.r, color.g, color.b = BossPanelUtils.GetThresholdColor(numItems / numNeeded)
								else
									-- This quest doesn't involve items.  Let's leave it grey.  We're lazy.
									color = {r=0.5,g=0.5,b=0.5}
								end
								
								if qtype == 'item' then
									items[itemName] = {quest = strQuestLogTitleText, item = itemName, needed = string.format("%s/%s",numItems,numNeeded), color=color}
								elseif qtype == 'monster' then
									mobs[mobName] = {quest = strQuestLogTitleText, mob = mobName, needed = string.format("%s/%s",numItems,numNeeded), color=color}
								end
								
								if done then
									done = format("(%s)",self.loc.QUEST_DONE)
								elseif numItems and numNeeded then
									done = string.format("%s/%s",numItems,numNeeded)
								else
									--I think this shouldn't happen.  ...but I haven't seen every quest out there.
									done = ""
								end
								
								lb[ii] = {description=desc, done=done, color=color}
							end
						end
						q.leaderboard = lb
						q.id = quest
						
						table.insert(quests[zoneIndex].quests,q)
					end
				end
			end
			
			self.quests = quests
			self.items = items
			self.mobs = mobs
			self.numQuests = numQuests
			self.numEntries = numEntries
			
			SelectQuestLogEntry(startingQuestLogSelection)
		end
	end,
	
	UpdateText = function(self)
		if not self.firstQuestLogUpdate then
			self:SetText("Scanning...")
		else
			local doneText = format("(%s)",self.loc.QUEST_DONE)
			local complete = 0
			for _, zone in self.numQuests do
				for _, quest in zone do
					if quest.complete == doneText then
						complete = complete + 1
					end
				end
			end
			local color = BossPanelUtils.GetThresholdHexColor(1 - (complete / self.numQuests))
			
			self:SetText(BossPanelUtils.Colorize(color,format("%d/%d", complete, self.numQuests)))
		end
	end,
	
	--Some credit to TheFly
	UpdateTooltip = function(self)
		local currentZone = GetZoneText()
		local cat, qi, height = 0, 0, 40
		self.numBut = 0
		
		if self.firstQuestLogUpdate and self.numQuests > 0 then
			for _,zone in self.quests do
				if self:IsShowingArea() and ((currentZone == zone.zone) or not self:IsShowingCurrentAreaOnly()) then
					cat = cat + 1
					height = height + self:AddElement(BossPanelUtils.Colorize("bfffbf",zone['zone']), "", 20+cat, self.charData.hidden[zone.zone])
				end
				for _,quest in zone.quests do
					if not self.charData.hidden[zone.zone] and ((currentZone == zone.zone) or not self:IsShowingCurrentAreaOnly()) then
						local thisQuest
						local questLevel = ""
						
						if self:IsShowingLevels() then
							questLevel = format("[%s%s] ", quest.level, quest.tag)
						end
						
						thisQuest = questLevel..quest.title
						
						local thisQuestColor = {r=1,g=1,b=1}
						if self:IsShowingDifficulty() then
							thisQuestColor = GetDifficultyColor(quest.level)
						end
						
						local leftHeader = "  "..self:AddColor(thisQuest, thisQuestColor)
						local rightHeader = (quest.complete or "\n")
						local left, right
						
						if self:IsShowingDescription() and ((currentZone == zone.zone) or not self:IsShowingCurrentAreaDescriptionOnly()) then
							if table.getn(quest.leaderboard) > 0 then
								for _,goal in quest.leaderboard do
									if not (string.find(goal.done, self.loc.QUEST_DONE) and not self:IsShowingCompletedObjectives()) then
										if self:IsColoringObjectives() then
											color = goal.color
										else
											color = thisQuestColor
										end
										left = (left and left.."\n" or "") .. self:AddColor(goal.description, color)
										right = (right and right.."\n" or "") .. self:AddColor((goal.done or ""), color)
									end
								end
							elseif not (quest.complete and not self:IsShowingCompletedObjectives()) then
								left = BossPanelUtils.Colorize("7f7f7f", quest.objective)
							end
						end
						qi = qi + 1
						height = height + self:AddElement(leftHeader, rightHeader, qi, IsQuestWatched(quest.id), left, right)
					end
				end
			end
		end
		local i = cat + qi + 1
		local button = getglobal("BossPanel_Quest"..i)
		while (button) do
			button:Hide()
			i = i + 1
			button = getglobal("BossPanel_Quest"..i)
		end
		BossPanel_QuestsFrame:SetHeight(height)
	end,
	
	--Credit to TheFly
	AddButton = function(self, num)
		local prev = getglobal("BossPanel_Quest"..(num-1))
		local button = CreateFrame("Button", "BossPanel_Quest"..num, BossPanel_QuestsFrame)
		local check = button:CreateTexture("BossPanel_Quest"..num.."Check", "ARTWORK")
		local hleft = button:CreateFontString("BossPanel_Quest"..num.."HLeft", "ARTWORK")
		local hright = button:CreateFontString("BossPanel_Quest"..num.."HRight", "ARTWORK")
		local left = button:CreateFontString("BossPanel_Quest"..num.."Left", "ARTWORK")
		local right = button:CreateFontString("BossPanel_Quest"..num.."Right", "ARTWORK")
		check:SetPoint("TOPLEFT", button)
		check:SetWidth(12)
		check:SetHeight(12)
		check:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
		hleft:SetFontObject(GameTooltipText)
		hleft:SetPoint("TOPLEFT", 13, 1)
		hleft:SetJustifyH("LEFT")
		hleft:SetWidth(200)
		hleft:Show()
		hright:SetFontObject(GameTooltipTextSmall)
		hright:SetPoint("TOPRIGHT", -15, 1)
		hright:SetJustifyH("RIGHT")
		hright:Show()
		left:SetFontObject(GameTooltipTextSmall)
		left:SetPoint("TOPLEFT", hleft, "BOTTOMLEFT", 15, 0)
		left:SetJustifyH("LEFT")
		left:SetWidth(185)		
		right:SetFontObject(GameTooltipTextSmall)
		right:SetPoint("TOPRIGHT", hright, "BOTTOMRIGHT")
		right:SetJustifyH("RIGHT")
		if prev then
			button:SetPoint("TOPLEFT", prev, "BOTTOMLEFT")
		else
			button:SetPoint("TOPLEFT", 3, -30)
		end
		button:SetWidth(275)
		button:SetScript("OnClick", function() self:Quest_Click() end)
		button:SetScript("OnEnter", function() self:Quest_Enter() end)
		button:SetScript("OnLeave", function() self:Quest_Leave() end)
		button:Hide()
		return button
	end,
	
	--Credit to TheFly
	AddElement = function(self, Hleft, Hright, id, checked, left, right)
		self.numBut = self.numBut + 1
		local i = self.numBut
		local button = getglobal("BossPanel_Quest"..i) or self:AddButton(i)
		if checked then
			getglobal("BossPanel_Quest"..i.."Check"):Show()
		else
			getglobal("BossPanel_Quest"..i.."Check"):Hide()
		end
		getglobal("BossPanel_Quest"..i.."HLeft"):SetText(Hleft)
		getglobal("BossPanel_Quest"..i.."HRight"):SetText(Hright or "")
		getglobal("BossPanel_Quest"..i.."HRight"):SetHeight(getglobal("BossPanel_Quest"..i.."HLeft"):GetHeight())
		local height = getglobal("BossPanel_Quest"..i.."HLeft"):GetHeight()+2
		if left then
			getglobal("BossPanel_Quest"..i.."Left"):SetText(left)
			getglobal("BossPanel_Quest"..i.."Right"):SetText(right or "")
			getglobal("BossPanel_Quest"..i.."Left"):Show()
			getglobal("BossPanel_Quest"..i.."Right"):Show()
			height = height + getglobal("BossPanel_Quest"..i.."Left"):GetHeight()
		else
			getglobal("BossPanel_Quest"..i.."Left"):Hide()
			getglobal("BossPanel_Quest"..i.."Right"):Hide()
		end
		button:SetHeight(height)
		button:SetID(id)
		button:Show()
		return height
	end,
	
	OnClick = function(self)
		ToggleQuestLog()
	end,
	
	AddColor = function(self, text, color)
		return BossPanelUtils.Colorize(format("%02X%02X%02X", color.r * 255, color.g * 255, color.b * 255),text)
	end,
})

BossPanel_Quests:RegisterForLoad()
